﻿/** 
 * ---------------------------------------------------------------------------
 * LapisItemUtility
 * LapisServer アイテムユーティリティクラス
 * ---------------------------------------------------------------------------
 * Rev.1  2017/06/06
 *   変更履歴
 *     ・自然染料　マゼンタ、ダークイエロー、ブライトメタルの問題改修
 *     ・家具関連の染色条件を変更
 */
using System;
using Server.Multis;

namespace Server.Items
{
    public class LapisItemUtility
    {
        private LapisItemUtility() { }

        /** デバッグフラグ */
        public static bool isDebug = false;

        // -------------------------------------------------------------------------------------------------------
        // LapisPigmentSystem関連
        // -------------------------------------------------------------------------------------------------------
        /**  布色 */
        public static readonly int[] CstUncutClothHue =
        {
            0x0483, 0x0488, 0x048c, 0x048a, // L1 Green highlight, Dark blue green, green blue, blue green
            0x0495, 0x048b, 0x0486, 0x0485, // L2 bright blue, purple blue, dark purple, dark red
            0x048d, 0x0490, 0x048e, 0x0491, // L3 ice light, puple light, fire light, brown light
            0x048f, 0x0494, 0x0484, 0x0497, // L4 green light, blue red, dark gray, jet black
            0x047f, 0x0482,                 // L5 ice green, ice blue
        };

        /** 布色名称 */
        public static readonly String[] CstUncutClothName =
        {
            "GreenHighlight","DarkBlueGreen","GreenBlue" ,"BlueGreen",
            "BrightBlue"    ,"PurpleBlue"   ,"DarkPurple","DarkRed",
            "IceLight"      ,"PupleLight"   ,"FireLight" ,"BrownLight",
            "GreenLight"    ,"BlueRed"      ,"DarkGray"  ,"JetBlack",
            "IceGreen"      ,"IceBlue"
        };
        /** 自然染料色 
         *    Rev.1 マゼンタ色コード修正
         *          ダークイエロー色コード修正
         *          ブライトメタル追加
         */
        public static readonly int[] CstNaturalHue =
        {
            0x0835,                         // Plain
            0x0674, 0x066e, 0x0475, 0x014f, // Red      BrightRed    DarkRed    IceRed
            0x084a, 0x051e, 0x0525, 0x0482, // Blue     BrightBlue   DarkBlue   IceBlue
            0x084d, 0x00fd, 0x08a9, 0x0038, // Yellow   BrightYellow DarkYellow IceYellow
            0x0850, 0x029f, 0x0591, 0x0105, // Green    BrightGreen  DarkGreen  IceGreen
            0x0468, 0x05dd, 0x05e5, 0x015a, // Orange   BrightOrange DarkOrange IceOrange
            0x000f, 0x013c, 0x04e6, 0x01ff, // Purple   BrightPurple DarkPurple IcePurple
            0x047e, 0x0497,                 // White    Black
            0x02ea, 0x0976, 0x0451,         // OffWhite Metal BrightMetal
            0x048b, 0x0155, 0x054e, 0x0187  // Raremagenta RarePink RareFireRed RareAqua
        };
        public static readonly String[] CstNaturalName =
        {
            "Plain"      ,
            "Red"        ,"BrightRed"   ,"DarkRed"    ,"IceRed"   ,
            "Blue"       ,"BrightBlue"  ,"DarkBlue"   ,"IceBlue"  ,
            "Yellow"     ,"BrightYellow","DarkYellow" ,"IceYellow",
            "Green"      ,"BrightGreen" ,"DarkGreen"  ,"IceGreen" ,
            "Orange"     ,"BrightOrange","DarkOrange" ,"IceOrange",
            "Purple"     ,"BrightPurple","DarkPurple" ,"IcePurple",
            "White"      ,"Black"       ,
            "OffWhite"   ,"Metal"       ,"BrightMetal"            ,
            "Raremagenta","RarePink"    ,"RareFireRed","RareAqua"
        };
        /** ToT AF色 */
        public static readonly int[] CstToT1AFHue =
        {
            0x0501, // ParagonGold
            0x0486, // VioletCouragePurple
            0x04f2, // InvulnerabilityBlue
            0x047E, // LunaWhite
            0x048F, // DryadGreen
            0x0455, // ShadowDancerBlack
            0x0021, // BerserkerRed
            0x058C, // NoxGreen
            0x066C, // RumRed
            0x054F  // FireOrange
            
        };
        /** ToT2 AF色名称 */
        public static readonly String[] CstToT1AFName =
        {
            "ParagonGold","VioletCouragePurple","InvulnerabilityBlue","LunaWhite",
            "DryadGreen","ShadowDancerBlack","BerserkerRed","NoxGreen","RumRed","FireOrange"
        };
        /** ToT2 AF色 */
        public static readonly int[] CstToT2AFHue =
        {
            0x096A, // Fadedcoal
            0x096B, // Coal
            0x0972, // FadedGold
            0x0977, // StormBronze
            0x097C, // Rose
            0x096C, // MidnightCoal
            0x0975, // FadedBronze
            0x097B, // FadedRose
            0x097E  // DeepRose
            
        };
        /** ToT2 AF色名称 */
        public static readonly String[] CstToT2AFName =
        {
            "Fadedcoal","Coal","FadedGold","StormBronze","Rose",
            "MidnightCoal","FadedBronze","FadedRose","DeepRose"
        };
        /** ToT2 TMAF色 */
        public static readonly int[] CstToT2TMAFHue =
        {
            0x0961, 0x0962, 0x0963, 0x0964, 0x0965, 0x0966,
            0x0967, 0x0968, 0x0969, 0x096a, 0x096b, 0x096c,
            0x096d, 0x096e, 0x096f, 0x0970, 0x0971, 0x0972,
            0x0973, 0x0974, 0x0975, 0x0976, 0x0977, 0x0978,
            0x0979, 0x097a, 0x097b, 0x097c, 0x097d, 0x097e
        };
        /** ToT2 TMAF色名称 */
        public static readonly String[] CstToT2TMAFName =
        {
            "ToT2 0x0961", "ToT2 0x0962", "ToT2 0x0963", "ToT2 0x0964", "ToT2 0x0965", "ToT2 0x0966",
            "ToT2 0x0967", "ToT2 0x0968", "ToT2 0x0969", "ToT2 0x096a", "ToT2 0x096b", "ToT2 0x096c",
            "ToT2 0x096d", "ToT2 0x096e", "ToT2 0x096f", "ToT2 0x0970", "ToT2 0x0971", "ToT2 0x0972",
            "ToT2 0x0973", "ToT2 0x0974", "ToT2 0x0975", "ToT2 0x0976", "ToT2 0x0977", "ToT2 0x0978",
            "ToT2 0x0979", "ToT2 0x097a", "ToT2 0x097b", "ToT2 0x097c", "ToT2 0x097d", "ToT2 0x097e"
        };
        /** ToT3 TMAF色 */
        public static readonly int[] CstToT3TMAFHue =
        {
            0x02E, // PaleOrange
            0x4B9, // FreshRose
            0x005, // ChaosBlue
            0x3E9, // Silver
            0x227, // NobleGold
            0x1C8, // LightGreen
            0x24F, // PaleBlue
            0x145, // FreshPlum
            0x3F0, // DeepBrown
            0x41A, // BurntBrown
        };
        /** ToT3 TMAF色名称 */
        public static readonly String[] CstToT3TMAFName =
        {
            "PaleOrange","FreshRose","ChaosBlue","Silver","NobleGold",
            "LightGreen","PaleBlue","FreshPlum","DeepBrown","BurntBrown"
        };
        /** 特殊色 */
        public static int[] CstSpecialHue =
        {
            0x0000, // 漂白剤
            0x000B, // シギル
            0x002B, // ハロウィン
            0x0480  // アイス
        };
        /** 特殊色名称 */
        public static String[] CstSpecialName =
        {
            "漂白剤",
            "シギル",
            "ハロウィン",
            "アイス"
        };
        /** デバッグメッセージ出力*/
        public static void dMsg(String msg)
        {
            if (isDebug)
            {
                Console.WriteLine(msg);
            }
        }
        /**
         * 染色の可否を返します
         */
         public static bool isDyable(Item useItem, Item dyeableItem, Mobile from)
        {
            // 布系はあかん
            if(dyeableItem is UncutCloth  || dyeableItem is Cloth  ||
               dyeableItem is BoltOfCloth || dyeableItem is Cotton || dyeableItem is Wool ||
               dyeableItem is Flax        || dyeableItem is BaseClothMaterial)
            {
                from.SendMessage("布または布作成の資材は染められません。");
                return false;
            }

            bool valid = (dyeableItem is IDyable      ||
                          dyeableItem is BaseBook     || dyeableItem is BaseClothing  ||
                          dyeableItem is BaseJewel    || dyeableItem is BaseStatuette ||
                          dyeableItem is BaseWeapon   || dyeableItem is BaseArmor     ||
                          dyeableItem is Runebook     || dyeableItem is Spellbook     ||
                          dyeableItem is ILapisDyable || dyeableItem is DecorateCoin
                          );

            // 家具関連
            if (!valid && FurnitureAttribute.Check(dyeableItem))
            {
                // Rev.1 染色条件変更
                // バックパック内に対象が存在する
                if (dyeableItem.IsChildOf(from.Backpack)){
                    valid = true;
                }
                else if (!from.InRange(useItem.GetWorldLocation(), 1) || !from.InRange(dyeableItem.GetWorldLocation(), 1))
                {
                    from.SendLocalizedMessage(500446); // That is too far away.
                    return false;
                }
                else
                {
                    BaseHouse house = BaseHouse.FindHouseAt(dyeableItem);

                    if (house == null)
                    {
                        from.SendLocalizedMessage(501022); // Furniture must be locked down to paint it.
                        return false;
                    }
                    else if (!house.IsCoOwner(from))
                    {
                        from.SendLocalizedMessage(501023); // You must be the owner to use this item.
                        return false;
                    }
                    else
                        valid = true;
                }
            }
            if (!valid)
            {
                // マネージャで追加したリストから検索
                if(LapisDyableManager.m_DyableList != null && LapisDyableManager.m_DyableList.Count != 0)
                {
                    for(int i = 0; i < LapisDyableManager.m_DyableList.Count; i++)
                    {
                        Type listType = LapisDyableManager.m_DyableList[i].GetType();
                        Type dyableItemType = dyeableItem.GetType();
                        if (listType == dyableItemType)
                        {
                            valid = true;
                            break;
                        }
                    }
                }
            }
            if (valid == false)
            {
                from.SendMessage("選択されたアイテムは染めることが出来ません。");
            }
            return valid;
        }
        /**
         * 死体湧きトクノ染料(TMAF2)ピグメントチケットを生成します
         */
        public static BasePigmentTicket makeRandomToT2Ticket()
        {
            BasePigmentTicket bpt = null;
            bpt = new TMAF2PigmentTicket();
            int index = Utility.Random(CstToT2TMAFHue.Length);
            bpt.PigCategory = 8;
            bpt.Hue = CstToT2TMAFHue[index];
            bpt.PigName = CstToT2TMAFName[index];
            bpt.PigCount = 1;
            return bpt;
        }
        /**
         * 死体湧きトクノ染料(TMAF3)ピグメントチケットを生成します
         */
        public static BasePigmentTicket makeRandomToT3Ticket()
        {
            BasePigmentTicket bpt = null;
            bpt = new TMAF3PigmentTicket();
            int index = Utility.Random(CstToT3TMAFHue.Length);
            bpt.PigCategory = 9;
            bpt.Hue = CstToT3TMAFHue[index];
            bpt.PigName = CstToT3TMAFName[index];
            bpt.PigCount = 1;
            return bpt;
        }
    }
    /**
     * LapisServer オリジナル染色可能アイテムI/Fクラス
     */
    public interface ILapisDyable
    {
    }
}
